
import enum


class Attribute(enum.Enum):
  RESET = 0
  BOLD = 1
  FAINT = 2
  ITALIC = 3
  UNDERLINE = 4
  SLOW_BLINK = 5
  RAPID_BLINK = 6
  REVERSE_VIDEO = 7
  CONCEAL = 8
  CROSSED_OUT = 9
  FONT_0 = 10
  FONT_1 = 11
  FONT_2 = 12
  FONT_3 = 13
  FONT_4 = 14
  FONT_5 = 15
  FONT_6 = 16
  FONT_7 = 17
  FONT_8 = 18
  FONT_9 = 19
  FRAKTUR = 20
  DOUBLY_UNDERLINE = 21
  NORMAL_INTENSITY = 22
  NOT_ITALIC = 23
  UNDERLINE_OFF = 24
  BLINK_OFF = 25
  REVERSE_OFF = 27
  REVEAL = 28
  CROSSED_OUT_OFF = 29
  FRAMED = 51
  ENCIRCLED = 52
  OVERLINED = 53
  FRAMED_OFF = 54
  OVERLINED_OFF = 55
